***************************************************************************************************************;
***************************************************************************************************************;
* Program name: New_Basu_v4_JAR.sas ;
***************************************************************************************************************;

*TO BE RUN IN THE LOCAL COMPUTER AND UPLOADED TO WRDS HOME DIRECTORY;
****** Prepare file with permnos and gvkeys FOR ANNUAL DATA ****************;
* File AT.Compdata_perm was downloaded from WRDS. It contains raw Compustat variables;

data AT.PERMNOGVKEY2017 (keep=gvkey FYEAR permno datadate bgdt);
set AT.Compdata_perm (keep=GVKEY DATADATE FYEAR PERMNO INDFMT obs=900000 where=(indfmt="INDL"));
*This step sets up the beginning and ending dates for user-defined windows;
format bgdt retbgdt3 retenddt3 date9.;
bgdt=intnx('month',DATADATE,-11);
retbgdt3=intnx('month',DATADATE,-8);
retenddt3=intnx('month',retbgdt3,11,'end');
if missing(permno)=0;
proc sort nodupkey;
by permno datadate;
run;

* COMPUTE ANNUAL BID-ASK SPREAD, BUT EVENTUALLY IT WILL NOT BE USED;
* COMPUTE ANNUAL VOLATILITY, DEFINED AS THE STANDARD DEVIATION OF ONE YEAR OF DAILY STOCK RETURNS;
* IF YOU RUN THE FOLLOWING CODE IN WRDS, THE UNIX COMMAND IS:  nohup get_bas_dvolat_2017.sas &  ;
************************************************************************************************;
%let wrds = wrds.wharton.upenn.edu 4016;options comamid=TCP;signon wrds username=_prompt_;

RSUBMIT;
options errors=1 noovp;
options nocenter ps=max ls=78;
options mprint source nodate symbolgen macrogen;
options msglevel=i;

libname mydata '/home/iese/fpa';
*Upload to home directory the table with the link PERMNO-GVKEY ;
proc upload data=AT.PERMNOGVKEY2017 out=mydata.PERMNOGVKEY2017;
data mydata.permnos;
set mydata.PERMNOGVKEY2017;
if FYEAR>1961;
proc sql;
  create table bas_dvolat as
  select co.*, std(ret) as dvolat, n(ret) as n_dvolat, 
               mean(abs(abs(askhi)-abs(bidlo))/( (abs(askhi)+abs(bidlo))/2 )) as bas, n(askhi) as n_bas
  from mydata.permnos as co, crsp.dsf (keep=permno date ret askhi bidlo where=(askhi ne 0 & bidlo ne 0))  as cr 
  where cr.permno=co.permno
  and co.bgdt <= cr.date <= co.datadate
  group by  co.permno, co.datadate;
quit;
proc sort data=bas_dvolat nodupkey out=mydata.rawbasdvolat2017;
by gvkey fyear;

proc download data=mydata.rawbasdvolat2017 out=AT.rawbasdvolat2017;
run;

ENDRSUBMIT;

************************************************************************************************;
*TO BE RUN IN THE LOCAL COMPUTER;
data AT.bas_stdret_2017;
set AT.rawbasdvolat2017;
if n_bas lt 125 then bas=.;
if n_dvolat lt 125 then dvolat=.;
stdret=dvolat;
drop bgdt n_bas n_dvolat dvolat;
if bas ne . or dvolat ne .;
proc means n mean std min p1 p50 p99 max;
var bas stdret;
run;

***********************************************************************************************;
***********************************************************************************************;

* COMPUTE LITIGATION PROX BASED ON MODEL (2) IN TABLE 7 OF KIM & SKINNER (2012 JAE); 

*This step pulls data from compustat and constructs initial variables;
* File AT.Compdata_perm was downloaded from WRDS. It contains raw Compustat variables;
data comp0 (keep=gvkey permno FYEAR datadate retbgdt0 FPS LNASSETS SALESGR SIC CSHO_lag) ;
set AT.compdata_perm (where=(INDFMT="INDL" and (1987 le FYEAR) and PRCC_F>0 and CSHO>0 and AT>0) obs=900000); 

* Drop financial firms and regulated industries;
if sic>5999 & sic<7000 or sic>=4900 & sic<=4949 then delete ; 

FPS=(2833<=SIC<=2836 or 8731<=SIC<=8734 or 7371<=SIC<=7379 or 3570<=SIC<=3577 or 3600<=SIC<=3674);
LNASSETS=log(AT);
if gvkey=lag(gvkey) and (fyear-1)=lag(fyear) then m1=1; else m1 = .;
SALE_lag=lag(SALE)*m1;
if SALE_lag>0 then SALESGR=(SALE-SALE_lag)/SALE_lag;
CSHO_lag=lag(CSHO)*m1;

*This step sets up the beginning and ending dates for monthly return windows;
format retbgdt0 date9.;
retbgdt0=intnx('month',DATADATE,-11); 

* COMPUTE RETURNS VARIABLES;
* FILE GLOPREM.CRSP2019 WAS DOWNLOADED FROM WRDS AND IT CONTAINS CRSP MONTHLY RETURNS;
proc sql;
  create table  ret_vars as
  select co.*, exp(sum(log(1+ret)))-1 as RET, n(ret) as n_periods, std(ret) as RET_STDEV, sum(vol)/10000 as turnovr
  from comp0 as co, GLOPREM.CRSP2019 as cr 
  where cr.permno=co.permno
  and co.retbgdt0 <= cr.date <= co.datadate
  group by  co.permno, co.datadate;
quit;
proc sort nodupkey;
by gvkey FYEAR;
run;

* COMPUTE MORE RETURNS VARIABLES;
proc sql;
  create table  skwdata as
  select co.*, cr.date, cr.ret
  from comp0 as co, GLOPREM.CRSP2019 as cr 
  where cr.permno=co.permno
  and co.retbgdt0 <= cr.date <= co.datadate
  order by  co.permno, co.fyear, cr.date;
quit;

proc means noprint;
by permno fyear;
var ret;
output out=sumstat skew=RET_SKW ;
run;

*MERGE FILES;
proc sql;
  create table final as
  select co.*, cr.RET_SKW, co.turnovr/co.CSHO_lag as TURNOVER
  from ret_vars as co left join sumstat as cr 
  on cr.permno=co.permno
  and co.fyear= cr.fyear
  order by  co.gvkey, co.fyear;
quit;
proc sort nodupkey;
by gvkey FYEAR;
run;

data final (keep=gvkey permno fyear datadate FPS RET RET_STDEV RET_SKW TURNOVER RET_lag RET_STDEV_lag 
				 RET_SKW_lag TURNOVER_lag SALESGR_lag LNASSETS_lag) ; 
set final;
if gvkey=lag(gvkey) and (fyear-1)=lag(fyear) then m1=1; else m1 = .;
RET_STDEV=RET_STDEV/sqrt(252/4);
RET_lag=lag(RET)*m1;
RET_STDEV_lag=lag(RET_STDEV)*m1;
RET_SKW_lag=lag(RET_SKW)*m1;
TURNOVER_lag=lag(TURNOVER)*m1;
SALESGR_lag=lag(SALESGR)*m1;
LNASSETS_lag=lag(LNASSETS)*m1;
label RET_SKW= ;
* WINSORIZE OR TRIM VARIABLES ANNUALLY AT THE 1 AND 99 PERCENTILES;
%winsor(dsetin=final, dsetout=final, vars=LNASSETS_lag SALESGR_lag RET RET_lag RET_SKW RET_SKW_lag RET_STDEV RET_STDEV_lag TURNOVER TURNOVER_lag   , type=winsor, pctl=1 99);
proc sort nodupkey;
by gvkey fyear;
run;

* COMPUTE THE PROBABILITY OF LITIGATION WITH MODEL (2) IN TABLE 7 OF KIM & SKINNER (2012 JAE) MEASURING VARIBALES AT THE START OF THE YEAR;;
data AT.litig;
set final;
z=-7.718 +0.180*FPS + 0.463*LNASSETS_lag +0.553*SALESGR_lag -0.498*RET_lag -0.359*RET_SKW_lag +14.437*RET_STDEV_lag +0.0004*TURNOVER_lag;
plit2_lag=1/(1+exp(-z));
run;
***********************************************************************************************;
***********************************************************************************************;

*This step pulls data from compustat and constructs initial variables. Some of them will not be used eventually;
* File AT.Compdata_perm was downloaded from WRDS. It contains raw Compustat variables;
data comp0 (drop=COGS INVT AP INVT_lag1 AP_lag1 m1) ;
set AT.compdata_perm (where=(INDFMT="INDL" and (1987 le FYEAR) and PRCC_F>0 and CSHO>0) obs=900000); 

SIC2=substr(SIC,1,2)*1; 
GICS=substr(GIND,1,2)*1;  
if CAPX='.' then CAPX=0;
if AQC='.' then AQC=0;
if SPPE='.' then SPPE=0;
if XIDOC='.' then XIDOC=0;
if XIDO='.' then XIDO=0;
if FOPO='.' then FOPO=0;
if TXBCO='.' then TXBCO=0;
if STKCO='.' then STKCO=0;
if missing(IDIT)=1 then IDIT=0;
if missing(MII)=1 then MII=0;
if missing(XINT)=1 then XINT=0;
if missing(RECTA)=1 then RECTA=0;
if missing(MSA)=1 then MSA=0;

if gvkey=lag(gvkey) and (fyear-1)=lag(fyear) then m1=1; else m1 = .;
RECT_lag1=lag(RECT)*m1;
INVT_lag1=lag(INVT)*m1;
AP_lag1=lag(AP)*m1;
if (COGS-INVT+INVT_lag1)>0 and SALE>0 and COGS>0 then
  CYCLE=360*((RECT+RECT_lag1)/2)/SALE + 360*((INVT+INVT_lag1)/2)/COGS - 360*((AP+AP_lag1)/2)/(COGS-INVT+INVT_lag1);
CYCLE_lag=lag(CYCLE)*m1;
AJEX_lag1=lag(AJEX)*m1;
LSALE=lag(SALE)*m1;
begA=lag(AT)*m1;
begNI=lag(NI)*m1;
LEMP=lag(EMP)*m1;;
begCEQ=lag(CEQ)*m1;
XIDOC_lag=lag(XIDOC)*m1;
LRECTA=lag(RECTA)*m1;
LMSA=lag(MSA)*m1;
lagAT=lag(AT)*m1;
PPEGT_lag=lag(PPEGT)*m1;

* Define Marginal rate as top statutory federal rates plus 0.02 average state tax rate;
If Fyear>=1988 and Fyear <=1992 then Mtaxrate=0.34+0.02;
If Fyear>=1993 then Mtaxrate=0.35+0.02;

* Define remaining variables as in the Nissim & Penman (2001) appendix. The name is the name in parentheses in the paper, joining separated words;
CoreNFE=XINT*(1-Mtaxrate) + DVP - IDIT*(1-Mtaxrate);
UFE=LMSA-MSA;
NFE=CoreNFE+UFE;
CSA=MSA-LMSA+RECTA-LRECTA;
CNI=NI-DVP+CSA;
oi=NFE+CNI+MII;
mve=PRCC_F*CSHO;
oi_lag=lag(oi)*m1;

* Drop financial firms and regulated industries;
if sic>5999 & sic<7000 or sic>=4900 & sic<=4949 then delete ; 
*if GICS=40 or GICS=55 then delete; 

*This step sets up the beginning and ending dates for monthly return windows;
format retbgdt3 retenddt3 retbgdt0 date9.;
retbgdt3=intnx('month',DATADATE,-8);
retenddt3=intnx('month',retbgdt3,11,'end');
retbgdt0=intnx('month',DATADATE,-11); 
retbgdt60=intnx('month',DATADATE,-59); 
run;

******************************************************************************;
* GET DATA FOR DUTTA, PATATOUKAS & WANG (2020) VARIABLES;
* FILE AT.Funda1950_2017 CONTAINS ADDITIONAL RAW COMPUSTAT VARIABLES DOWNLOADED FROM WRDS;

data DPW (keep=gvkey fyear DPC RECCH INVCH APALCH TXACH AOLOCH TXDC ESUBC SPPIV IVNCF CCACC DA STACC OTH CFI OPACC CCACC_lag CCACCmve CCACCmve_lag INVT INVT_lag) ;
set AT.Funda1950_2017 (keep=gvkey fyear DPC RECCH INVCH APALCH TXACH AOLOCH TXDC ESUBC SPPIV IVNCF AT SPI XIDO PRCC_F CSHO INVT where=(AT>0 and 1987 le FYEAR));
if gvkey=lag(gvkey) and (fyear-1)=lag(fyear) then m1=1; else m1 = .;
lagAT=lag(AT)*m1;
INVT_lag=lag(INVT)*m1;
MVE=PRCC_F*CSHO;
MVE_lag=lag(MVE)*m1;
SPI_lag=lag(SPI)*m1;
XIDO_lag=lag(XIDO)*m1;
CCACC=sum(SPI, -XIDO, 0)/lagAT;
if CCACC>0 then CCACC=0;
DA=-sum(DPC, 0)/lagAT;
STACC=sum(-RECCH, -INVCH, -APALCH, -TXACH, -AOLOCH, 0)/lagAT; 
OTH=sum(-TXDC, -ESUBC, -SPPIV, 0)/lagAT;
CFI=sum(IVNCF, 0)/lagAT;
OPACC=SUM(CCACC, STACC, DA, OTH);
CCACC_lag=sum(SPI_lag, -XIDO_lag, 0)/lagAT;
if CCACC_lag>0 then CCACC_lag=0;
if MVE_lag>0 then CCACCmve=sum(SPI, -XIDO, 0)/MVE_lag;
if CCACCmve>0 then CCACCmve=0;
if MVE_lag>0 then CCACCmve_lag=sum(SPI_lag, -XIDO_lag, 0)/MVE_lag;
if CCACCmve_lag>0 then CCACCmve_lag=0;

proc sort nodupkey;
by gvkey fyear;
proc means mean std min p25 p50 p75 max n;
var CCACC CCACC_lag DA STACC OTH OPACC CCACCmve CCACCmve_lag;
run;
******************************************************************************;

*Merge in DPW;
data comp0;
merge comp0 (in=in1) DPW;
by gvkey fyear;
if in1;
run;


* Add the first date in CRSP to compute the age of the firm;
* The file "AT.crspbegdates" is obtained from the CRSP Stock Header Information;
data date1 (keep=permno firstdate);
set AT.crspbegdates;
firstdate=begprc;
format firstdate YYMMDDN8.;
run;
proc sql;
  create table  comp00 as
  select co.*, d.firstdate
  from comp0 as co, date1 as d 
  where co.permno=d.permno ;
quit;


* Add the bid-ask spread (BAS), the volatility of stock returns (STDRET), firm AGE and an inverse proxy of investment cycle length (INVCYCLE);
* Some of these variables eventually will not be used;
data comp0 (drop=firstdate m1);
merge comp00 (in=in1) AT.bas_stdret_2017 (drop=permno datadate);
by gvkey fyear;
if in1;
age=(datadate-firstdate)/365;
if age<0 then age =0;
if gvkey=lag(gvkey) and (fyear-1)=lag(fyear) then m1=1; else m1 = .;
if begA > 0 then INVCYCLE=DP/begA;
INVCYCLE_lag=lag(INVCYCLE)*m1;
BAS_lag=lag(BAS)*m1;
STDRET_lag=lag(STDRET)*m1;
run;

***************************************************************************************;
* COMPUTE ANNUAL RETURNS ENDING AT FYE;
* FILE AT.CRSP1950_2017 WAS DOWLOADED FROM WRDS AND IT CONTAINS RAW RETURNS;
proc sql;
  create table  cum_returns as
  select co.*, exp(sum(log(1+ret)))-1 as RET12, n(ret) as n_periods 
  from comp0 as co, AT.CRSP1950_2017 (keep=permno date ret)  as cr 
  where cr.permno=co.permno
  and co.retbgdt0 <= cr.date <= co.datadate
  group by  co.permno, co.datadate;
quit;
proc sort nodupkey;
by gvkey FYEAR;
run;

data comp0 ;
set cum_returns;
by gvkey FYEAR;
if n_periods=12 ;
run;

* COMPUTE ANNUAL RETURNS ENDING 3 MONTHS AFTER FYE (EVENTUALLY NOT USED);
proc sql;
  create table  cum_returns3 as
  select co.*, exp(sum(log(1+ret)))-1 as RET15, n(ret) as n_periods15 
  from comp0 as co, AT.CRSP1950_2017 (keep=permno date ret)  as cr 
  where cr.permno=co.permno
  and co.retbgdt3 <= cr.date <= co.retenddt3
  group by  co.permno, co.datadate;
quit;
proc sort nodupkey;
by gvkey FYEAR;
run;

data comp0 (drop=retbgdt3 retenddt3 retbgdt0 n_periods n_periods15 ret12 ret15);
set cum_returns3;
by gvkey FYEAR;
if n_periods15=12 THEN RET3=RET15 ;
RET=RET12;
proc means mean std skew min p1 p25 median p75 p99 max n;
var RET RET3 FYEAR;
run;
***************************************************************************************;

* Construct more variables of interest. Not all of them will be eventually used; 
data comp0;
set comp0;
MVE=PRCC_F*CSHO;
P=PRCC_F/AJEX;
DIV=DVPSX_F/AJEX;
EPS=EPSPX/AJEX;
SHARES=CSHO*AJEX;
BM=CEQ/MVE;
CC_ACC = min((-FOPO+TXBCO+STKCO),0) + min((XIDO-XIDOC),0);
if CEQ ne 0 then MB=MVE/CEQ;

MVE_lag=lag(MVE);
P_lag=lag(P);
EPS_lag=lag(EPS);
BM_lag=lag(BM);
MB_lag=lag(MB);
BV_lag=lag(CEQ);
SHARES_lag=lag(SHARES);
PRCC_F_lag=lag(PRCC_F);
CSHO_LAG=lag(CSHO);
CC_ACC_lag=lag(CC_ACC);
if gvkey ne lag(gvkey) or FYEAR-1 ne lag(FYEAR) then do;
MVE_lag='.';
P_lag='.';
EPS_lag='.';
BM_lag='.';
MB_lag='.';
BV_lag='.';
SHARES_lag='.';
PRCC_F_lag='.';
CSHO_LAG='.';
CC_ACC_lag='.';
end;  
if BM_lag<0 then delete;
R=(P+DIV-P_lag)/P_lag;

SIZE=log(MVE);
SIZE_lag=log(MVE_lag);
X=IB/MVE_lag;
CFO=(OANCF-XIDOC)/MVE_lag;
CFOA=(OANCF-XIDOC)/lagAT;
FCF=(OANCF-CAPX-AQC+SPPE)/MVE_lag;
ACC=X-CFO;
ACCFC=X-FCF;
if TXDB = . then TXDB=0;
LEVERAGE=sum(DLC, DLTT)/(AT+MVE-CEQ );
if AT>0 then LEVTA=sum(DLC, DLTT)/AT;
LEVMVE=sum(DLC, DLTT)/MVE;
if AT>0 then MBA=(AT+MVE-CEQ )/AT;
BMA=1/MBA;
MVA=AT+MVE-CEQ;

IB_lag=lag(IB);
OANCF_lag=lag(OANCF);
CAPX_lag=lag(CAPX);
AQC_lag=lag(AQC);
SPPE_lag=lag(SPPE);
LEVERAGE_lag=lag(LEVERAGE);
LEVTA_lag=lag(LEVTA);
LEVMVE_lag=lag(LEVMVE);
MBA_lag=lag(MBA);
BMA_lag=lag(BMA);
MVA_lag=lag(MVA);
CC_ACC_lag2=lag(CC_ACC_lag);
if gvkey ne lag(gvkey) or FYEAR-1 ne lag(FYEAR) then do;
IB_lag='.';
OANCF_lag='.';
CAPX_lag='.';
AQC_lag='.';
SPPE_lag='.';
LEVERAGE_lag='.';
LEVTA_lag='.';
LEVMVE_lag='.';
MBA_lag='.';
BMA_lag='.';
MVA_lag='.';
CC_ACC_lag2=".";
end;

CC_ACC=CC_ACC/MVE_lag;
CC_ACC_lag=CC_ACC_lag/MVE_lag;
CC_ACC_lag2=CC_ACC_lag2/MVE_lag;
CC_ACC_lag2=CC_ACC_lag2/MVE_lag;
CC_ACC2=CC_ACC/MVA_lag;
CC_ACC2_lag=CC_ACC_lag/MVA_lag; 

CEMP=EMP-LEMP;
CSALE=(SALE-LSALE);
IF MISSING(CSALE*CEMP)=0 THEN CURT=0;
IF CSALE<0 & CEMP<0 then CURT=1;

X_lag=IB_lag/MVE_lag;
CFO_lag=(OANCF_lag-XIDOC_lag)/MVE_lag;
CFOA_lag=(OANCF_lag-XIDOC_lag)/lagAT;
ACC_lag=X_lag-CFO_lag;
FCF_lag=(OANCF_lag-CAPX_lag-AQC_lag+SPPE_lag)/MVE_lag;
ACCFC_lag=(IB_lag-FCF_lag)/MVE_lag;

if missing(X_lag)=0 then IX=(X_lag<0);
IX_X_lag=IX*X_lag;

CHPRICE=RET*P_lag;
CHP=R*P_lag;
IBPS=IB/SHARES;
LIBPS=IB_lag/SHARES;

if MVE_lag>0 then inv_MVE_lag = 1/MVE_lag;
if MVE_lag>0 then CSALES_CAR = (SALE-LSALE)/MVE_lag + RECCH/MVE_lag;
if MVE_lag>0 then PPE = PPEGT/MVE_lag;
if begA>0 then LROA = begNI/begA;
if RECCH = . then CSALES_CAR = (SALE-LSALE)/MVE_lag - (RECT-RECT_lag1)/MVE_lag;     

mroa=oi/MVA_lag;
mroa_lag=oi_lag/MVA_lag;
acc1=mroa-(FCF*MVE_lag)/MVA_lag;
acc1_lag=mroa_lag-(FCF_lag*MVE_lag)/MVA_lag;
acc2=mroa-(CFO*MVE_lag)/MVA_lag;
acc2_lag=mroa_lag-(CFO_lag*MVE_lag)/MVA_lag;

*********************************;
CSALEMVE=CSALE/MVE_lag;
if CSALEMVE ne . then DS=0;
if CSALEMVE < 0 then DS=1;
DEP=DPC/MVE_lag;
INT=-(XINT-IDIT)/MVE_lag;
CHREC=-RECCH/MVE_lag;

EBITDA=(NI+XINT-IDIT+DP+TXT)/MVE_lag;
PERFCOV_X=OIBDP/MVE_lag;

DPC_lag=lag(DPC);
DP_lag=lag(DP);
XINT_lag=lag(XINT);
IDIT_lag=lag(IDIT);
RECCH_lag=lag(RECCH);
TXT_lag=lag(TXT);
OIBDP_lag=lag(OIBDP);
if gvkey ne lag(gvkey) or FYEAR-1 ne lag(FYEAR) then do;
DPC_lag='.';
DP_lag='.';
XINT_lag='.';
IDIT_lag='.';
RECCH_lag='.';
TXT_lag='.';
OIBDP_lag='.';
end; 

DEP_lag=DPC_lag/MVE_lag;
INT_lag=-(XINT_lag-IDIT_lag)/MVE_lag;
CHREC_lag=-RECCH_lag/MVE_lag;

EBITDA_lag=(begNI+XINT_lag-IDIT_lag+DP_lag+TXT_lag)/MVE_lag;
PERFCOV_X_lag=OIBDP_lag/MVE_lag;

ACCBFDP=sum(ACC, DEP);
ACCBFDP_lag=sum(ACC_lag, DEP_lag);
inv_price_lag=1/prcc_f_lag;
********************************;

************************;
if stdret = . then delete;
************************; 
proc means n mean std skew min p1 p25 p50 p75 p99 max;
var : ;
run;
*****************************************************************************************************;

* WINSORIZE OR TRIM VARIABLES ANNUALLY AT THE 1 AND 99 PERCENTILES;
%winsor(dsetin=comp0, dsetout=comp1, byvar=FYEAR, vars=X CFO ACC RET RET3 R X_lag CFO_lag mb mb_lag size size_lag leverage levta  
    bas stdret bas_lag stdret_lag LEVMVE CHPRICE CHP IBPS LIBPS FCF FCF_lag ACCFC ACCFC_lag LEVERAGE_lag LEVTA_lag LEVMVE_lag
    MBA MBA_lag BMA_lag MVA MVA_lag CYCLE CYCLE_lag INVCYCLE INVCYCLE_lag CSALES_CAR PPE LROA mroa mroa_lag acc1 acc1_lag acc2
    acc2_lag CC_ACC CC_ACC_lag CC_ACC_lag2 CC_ACC2 CC_ACC2_lag CSALEMVE DEP DEP_lag INT INT_lag CHREC CHREC_lag EBITDA EBITDA_lag  
	OIBDP OIBDP_lag PERFCOV_X PERFCOV_X_lag CFOA CFOA_lag ACCBFDP ACCBFDP_lag 
	CCACCmve CCACCmve_lag IX_X_lag , type=winsor, pctl=1 99);

*****************************************************************************************************;

* Estimate the unexpected earnings model of Ball, Kothari & Nikolaev (2013 TAR), with intercept;
* get data, requiring a minimum of 10 observations in an industry-group;

proc sql;
   create table compdata
   as select * 
   from comp1
   where X*IX*X_lag*IX_X_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_x;
by SIC2 FYEAR;
model X=IX X_lag IX_X_lag  /rsquare ;
output out=comp1 r=x_unex;
data par_x;
set par_x;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept IX X_lag IX_X_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept IX X_lag IX_X_lag n rsq ;
proc means data=par_x n mean t std min p25 p50 p75 max;
var Intercept IX X_lag IX_X_lag  n rsq;
run;

proc means data=comp1 n mean std skew min p1 p25 p50 p75 p99 max;
var x_unex fyear;
run;
******************************************************;
* ESTIMATE UNEXPECTED ACCRUALS. SOME OF THE NEXT UNEXPECTED VARIABLES WILL NOT BE USED EVENTUALLY;
* Estimate industry model of Patatoukas and Thomas (2016, footnote 15);
* Get data, requiring a minimum of 10 observations in an industry-year group;

* UACCFC;
proc sql;
   create table compdata
   as select * 
   from comp1
   where ACCFC*ACCFC_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj1;
by SIC2 FYEAR;
model ACCFC=ACCFC_lag /rsquare;
output out=temp_mj1 (keep=gvkey fyear UACCFC) r=UACCFC ;
proc sort data=temp_mj1 nodupkey;
by gvkey FYEAR;
run;
data par_mj1;
set par_mj1;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept ACCFC_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept ACCFC_lag n rsq;
proc means data=par_mj1 n mean t std min p25 p50 p75 max;
var Intercept ACCFC_lag n rsq;
proc means data=temp_mj1 n mean t std min p1 p25 p50 p75 p99 max;
var UACCFC;
run;

*****************************************************************************************************;

* UACC;
proc sql;
   create table compdata
   as select * 
   from comp1
   where ACC*ACC_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj2;
by SIC2 FYEAR;
model ACC=ACC_lag /rsquare;
output out=temp_mj2 (keep=gvkey fyear UACC) r=UACC ;
proc sort data=temp_mj2 nodupkey;
by gvkey FYEAR;
run;
data par_mj2;
set par_mj2;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept ACC_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept ACC_lag n rsq;
proc means data=par_mj2 n mean t std min p25 p50 p75 max;
var Intercept ACC_lag n rsq;
proc means data=temp_mj2 n mean t std min p1 p25 p50 p75 p99 max;
var UACC;
run;

*****************************************************************************************************;

* UCC_ACC;
proc sql;
   create table compdata
   as select * 
   from comp1
   where CC_ACC*CC_ACC_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj3;
by SIC2 FYEAR;
model CC_ACC=CC_ACC_lag /rsquare;
output out=temp_mj3 (keep=gvkey fyear UCC_ACC) r=UCC_ACC ;
proc sort data=temp_mj3 nodupkey;
by gvkey FYEAR;
run;
data par_mj3;
set par_mj3;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept CC_ACC_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept CC_ACC_lag n rsq;
proc means data=par_mj3 n mean t std min p25 p50 p75 max;
var Intercept CC_ACC_lag n rsq;
proc means data=temp_mj3 n mean t std min p1 p25 p50 p75 p99 max;
var UCC_ACC;
run;

*****************************************************************************************************;

* UACC2;
proc sql;
   create table compdata
   as select * 
   from comp1
   where ACC2*ACC2_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj4;
by SIC2 FYEAR;
model ACC2=ACC2_lag /rsquare;
output out=temp_mj4 (keep=gvkey fyear UACC2) r=UACC2 ;
proc sort data=temp_mj4 nodupkey;
by gvkey FYEAR;
run;
data par_mj4;
set par_mj4;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept ACC2_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept ACC2_lag n rsq;
proc means data=par_mj4 n mean t std min p25 p50 p75 max;
var Intercept ACC2_lag n rsq;
proc means data=temp_mj4 n mean t std min p1 p25 p50 p75 p99 max;
var UACC2;
run;
*****************************************************************************************************;

* UACCBFDP;
proc sql;
   create table compdata
   as select * 
   from comp1
   where ACCBFDP*ACCBFDP_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj5;
by SIC2 FYEAR;
model ACCBFDP=ACCBFDP_lag /rsquare;
output out=temp_mj5 (keep=gvkey fyear UACCBFDP) r=UACCBFDP ;
proc sort data=temp_mj5 nodupkey;
by gvkey FYEAR;
run;
data par_mj5;
set par_mj5;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept ACCBFDP_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept ACCBFDP_lag n rsq;
proc means data=par_mj5 n mean t std min p25 p50 p75 max;
var Intercept ACCBFDP_lag n rsq;
proc means data=temp_mj5 n mean t std min p1 p25 p50 p75 p99 max;
var UACCBFDP;
run;

*****************************************************************************************************;

* UCCACCmve;
proc sql;
   create table compdata
   as select * 
   from comp1
   where CCACCmve*CCACCmve_lag  ne .
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj6;
by SIC2 FYEAR;
model CCACCmve=CCACCmve_lag /rsquare;
output out=temp_mj6 (keep=gvkey fyear UCCACCmve) r=UCCACCmve ;
proc sort data=temp_mj6 nodupkey;
by gvkey FYEAR;
run;
data par_mj6;
set par_mj6;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept CCACCmve_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept CCACCmve_lag n rsq;
proc means data=par_mj6 n mean t std min p25 p50 p75 max;
var Intercept CCACCmve_lag n rsq;
proc means data=temp_mj6 n mean t std min p1 p25 p50 p75 p99 max;
var UCCACCmve;
run;

*****************************************************************************************************;

* UCC_ACC_lag;
proc sql;
   create table compdata
   as select * 
   from comp1
   where CC_ACC_lag*CC_ACC_lag2 ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj7;
by SIC2 FYEAR;
model CC_ACC_lag=CC_ACC_lag2 /rsquare;
output out=temp_mj7 (keep=gvkey fyear UCC_ACC_lag) r=UCC_ACC_lag ;
proc sort data=temp_mj7 nodupkey;
by gvkey FYEAR;
run;
data par_mj7;
set par_mj7;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept CC_ACC_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept CC_ACC_lag n rsq;
proc means data=par_mj7 n mean t std min p25 p50 p75 max;
var Intercept CC_ACC_lag n rsq;
proc means data=temp_mj7 n mean t std min p1 p25 p50 p75 p99 max;
var UCC_ACC_lag;
run;

*****************************************************************************************************;

* UCC_ACCalt;
proc sql;
   create table compdata
   as select *, (CC_ACC_lag<0) as IA, calculated IA*CC_ACC_lag as IA_CC_ACC_lag
   from comp1
   where CC_ACC*CC_ACC_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_mj8;
by SIC2 FYEAR;
model CC_ACC= IA CC_ACC_lag IA_CC_ACC_lag /rsquare;
output out=temp_mj8 (keep=gvkey fyear UCC_ACCalt) r=UCC_ACCalt ;
proc sort data=temp_mj8 nodupkey;
by gvkey FYEAR;
run;
data par_mj8;
set par_mj8;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR Intercept IA CC_ACC_lag IA_CC_ACC_lag n rsq label=" "; 
keep SIC2 FYEAR Intercept IA CC_ACC_lag IA_CC_ACC_lag n rsq;
proc means data=par_mj8 n mean t std min p25 p50 p75 max;
var Intercept IA CC_ACC_lag IA_CC_ACC_lag n rsq;
proc means data=temp_mj8 n mean t std min p1 p25 p50 p75 p99 max;
var UCC_ACCalt;
run;

*****************************************************************************************************;

* Estimate the CFO persistance model of Dutta & Patatoukas (2017 TAR), AR(1) model  ;
* By SIC2 FYEAR groups, requiring a minimum of 10 observations in each industry-year group;

proc sql;
   create table compdata
   as select * 
   from comp1
   where CFO*CFO_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_c;
by SIC2 FYEAR;
model CFO=CFO_lag  /rsquare ;
data par_c;
set par_c;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR  CFO_lag n rsq label=" "; 
keep SIC2 FYEAR  CFO_lag n rsq ;
proc means data=par_c n mean t std min p1 p25 p50 p75 p99 max;
var  CFO_lag n rsq;
run;
data cfo_sic2 (keep= SIC2 FYEAR w_cfo);
set par_c;
w_cfo=CFO_lag;
run;

*************************************************;

* Estimate the FCF persistance model of Dutta & Patatoukas (2017 TAR), AR(1) model  ;
* By SIC2 FYEAR groups, requiring a minimum of 10 observations in each industry-year group;

proc sql;
   create table compdata
   as select * 
   from comp1
   where FCF*FCF_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_c;
by SIC2 FYEAR;
model FCF=FCF_lag  /rsquare ;
data par_c;
set par_c;
n=sum(_edf_,_p_);
rsq=_RSQ_;
if n>=10;
attrib SIC2 FYEAR  FCF_lag n rsq label=" "; 
keep SIC2 FYEAR  FCF_lag n rsq ;
proc means data=par_c n mean t std min p1 p25 p50 p75 p99 max;
var  FCF_lag  n rsq;
run;
data fcf_sic2 (keep= SIC2 FYEAR w_fcf);
set par_c;
w_fcf=FCF_lag;
run;

proc sort data=comp1;
by SIC2 FYEAR;
data comp1;
merge comp1 (in=in1) cfo_sic2 fcf_sic2  ;
by SIC2 FYEAR;
if in1;
proc means mean std skew p10 p25 p50 p75 p90 n;
var w_cfo w_fcf;
run;

******************************************************;

* COMPUTE CFORES VARIABLE AS IN DUTTA, PATATOUKAS AND WANG (2020);
proc sql;
   create table compdatacfoa
   as select * 
   from comp1
   where CFOA*CFOA_lag ne .
   group by SIC2,fyear
   having n(gvkey) >= 10
   order by SIC2, FYEAR;
   quit;
proc reg data=compdata noprint outest=par_c;
by SIC2 FYEAR;
model CFOA=CFOA_lag  /rsquare ;
output out=CFORES (keep=gvkey fyear CFORES) r=CFORES ;
proc sort nodupkey data=CFORES;
by gvkey fyear;
run;
proc means mean std p25 p50 p75 ; var cfores; run;

******************************************************;

* Annual quintile portfolios of beginning-of-year SIZE;
proc sort data=comp1;
by FYEAR;
proc rank data=comp1 groups=5 out=comp1;
var SIZE_lag;
ranks QSIZE_lag ;
by FYEAR;
proc sort data=comp1;
by FYEAR QSIZE_lag ;

* Annual quintile portfolios of beginning-of-year SIZE and beginning-of-year BM;
proc sort data=comp1 ;
by FYEAR SIZE_lag ; run;
proc rank data=comp1 groups=5 out=comp2;
var BM_lag;
ranks QBM_lag ;
by FYEAR QSIZE_lag ;
run;

proc sort data=comp2;
by FYEAR QSIZE_lag QBM_lag; 
run;

* COMPUTE THE MEAN OF RET AND RET3 BY FYEAR, QSIZE_lag, QBM_lag;
* to estimate unexpected returns uret and uret3 ;
proc sql;
create table comp3 as
	  select *, sum(size_lag*RET)/sum(size_lag) as eret, sum(size_lag*RET3)/sum(size_lag) as eret3, RET-calculated eret as uret, RET3-calculated eret3 as uret3
      from comp2
      group by FYEAR, QSIZE_lag, QBM_lag
	  order by gvkey, fyear ;
quit;
run;

* COMPUTE THE SPREAD BETWEEN THE VARIANCE OF POSITIVE RETURNS AND NEGATIVE RETURNS AS IN DUTTA & PATATOUKAS (2016 TAR);
* Only for uret;
data comp3;
set comp3;
* Impose that unexpected returns cannot be lower than -1 ;
if uret<-1  and missing(uret)= 0 then uret= -0.99999;
if uret3<-1 and missing(uret3)=0 then uret3=-0.99999;
if uret>=0 then pos=uret ;
if uret<0  then neg =uret ;
proc sql;
create table comp3 as
      select *, var(pos) as varpos, var(neg) as varneg, calculated varpos-calculated varneg as spread
      from comp3
      group by FYEAR, QSIZE_lag, QBM_lag
	  order by gvkey, fyear ;
quit;
run;

proc means mean std skew min p1 p10 p25 p50 p75 p90 p99 max n;
var : ;
run;

proc means mean std skew p25 p50 p75 n;
var X ACC ACCFC CFO FCF x_unex uret eret w_cfo w_fcf spread;
run;

proc corr pearson spearman;
var X ACC ACCFC CFO FCF x_unex uret ;
run;

* Merge the estimates of unexpected accruals and CFORES;
data AT.sample2017JAR (drop=QSIZE_lag QBM_lag);
merge comp3 (in=in1) temp_mj1 temp_mj2 temp_mj3 temp_mj4 temp_mj5 temp_mj6 temp_mj7 temp_mj8 CFORES  ;
by gvkey fyear;
if in1;
if missing(X*ACC*ACCFC*CFO*FCF*x_unex*UACC*UCC_ACC*R*uret*eret*w_cfo*w_fcf*spread*mb_lag*size*leverage*levta*leverage_lag*levta_lag*LEVMVE*LEVMVE_lag)=1 then delete;
proc means mean std skew min p1 p25 p50 p75 p99 max n;
var X ACC ACCFC CFO FCF x_unex uret eret w_cfo w_fcf spread w_cfo w_fcf mb_lag size leverage levta UACCFC UACC CURT acc1 mroa mroa_lag oi oi_lag ;
run;
* WINSORIZE ADDITIONAL VARIABLES ANNUALLY AT THE 1 AND 99 PERCENTILES;
%winsor(dsetin=AT.sample2017JAR, dsetout=AT.sample2017JAR, vars= CFORES, byvar=fyear, type=winsor, pctl=1 99); 
proc sort nodupkey;  by gvkey fyear; 
run;
***********************************************************************************;

* MERGE PIN DATA (from DUARTE & YOUNG, 2009 JFE) ;
data data AT.sample2017JAR (drop=pin);
merge AT.sample2017JAR (in=in1) AT.Pins_1983_2004 (keep=gvkey fyear pin);
if in1;
by gvkey fyear;
pin2=pin;
proc means;
var pin2;
run;

*MERGE PPENT INTAN CAPX SPPE AQC ADDITIONAL VARIABLES;
*FILE AT.Funda1950_2017 CONTAINS ADDITIONAL RAW COMPUSTAT VARIABLES DOWNLOADED FROM WRDS;
data ADDLVARS;
set AT.Funda1950_2017 (keep=gvkey fyear PPENT INTAN CAPX SPPE AQC  where=(1988 le FYEAR)  );
proc sort nodupkey;
by gvkey fyear;
run;
data AT.sample2017JAR ;
merge AT.sample2017JAR (in=in1) ADDLVARS;
if in1;
by gvkey fyear;
run;

* MERGE LITIGATION PROXY WITH MAIN SAMPLE;
data AT.sample2017JAR ;
merge AT.sample2017JAR (in=in1) AT.litig (keep=gvkey fyear  plit2_lag );
if in1;
by gvkey fyear;
run;

* EXPORT SAMPLE TO STATA;
PROC EXPORT DATA= AT.sample2017JAR
            OUTFILE= "C:\RESEARCH\AT\Data\sample2017JAR.dta" 
            DBMS=STATA REPLACE;
RUN;


